/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CUDA streams - JGL - 10/11/2010
// 
// Juan Gómez-Luna*, José María González-Linares#, José Ignacio Benavides* and Nicolás Guil#
// *University of Córdoba, #University of Málaga (Spain)
// http://www.ac.uma.es/~vip/
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Streamed RGB to grayscale
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// streams_RGB2gray_gold.cpp
// This file contains the CPU code used in Section 4.3.
// This code is a variation of histogram256_gold.cpp to compute grayscale histogram.
//
// Reference:
// @article{GomezLuna20121117,
//   title = "Performance models for asynchronous data transfers on consumer Graphics Processing Units",
//   journal = "Journal of Parallel and Distributed Computing",
//   volume = "72",
//   number = "9",
//   pages = "1117 - 1126",
//   year = "2012",
//   note = "<ce:title>Accelerators for High-Performance Computing</ce:title>",
//   issn = "0743-7315",
//   doi = "10.1016/j.jpdc.2011.07.011",
//   url = "http://www.sciencedirect.com/science/article/pii/S0743731511001468",
//   author = "Juan G\'omez-Luna and Jos\'e Mar\'ia Gonz\'alez-Linares and Jos\'e Ignacio Benavides and Nicol\'as Guil",
// }
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define BIN_COUNT 256

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// RGB to gray CPU
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
extern "C" void rgb2grayCPU(
    unsigned int *h_Result,
    unsigned int *h_Data,
    int dataN
){
    int i;
    unsigned int data4;

    for(i = 0; i < BIN_COUNT; i++)
        h_Result[i] = 0;

    for (i = 0; i < dataN; i++){
        data4 = h_Data[i];
        unsigned int B = (data4 >>  0) & 0xFFU;
        unsigned int G = (data4 >>  8) & 0xFFU;
        unsigned int R = (data4 >>  16) & 0xFFU;

	h_Result[(unsigned int)(0.30*R + 0.59*G + 0.11*B)] += 1;
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
